/*
 Copyright (c) 2015, Apple Inc. All rights reserved.
 Copyright (c) 2015, Ricardo Sánchez-Sáez.
 
 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:
 
 1.  Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 2.  Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation and/or
 other materials provided with the distribution.
 
 3.  Neither the name of the copyright holder(s) nor the names of any contributors
 may be used to endorse or promote products derived from this software without
 specific prior written permission. No license is granted to the trademarks of
 the copyright holders even if such marks are included in this software.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "OCKDefines.h"
#import "OCKContact.h"


OCK_EXTERN NSString *const OCKErrorDomain OCK_AVAILABLE_DECL;

/// The exception thrown when an invalid argument is passed to a method or function.
OCK_EXTERN NSString *const OCKInvalidArgumentException OCK_AVAILABLE_DECL;

/// `ORKErrorCode` codes are used for errors in the domain `OCKErrorDomain`.
typedef NS_ENUM(NSInteger, OCKErrorCode) {
    /// No matching object was found.
    OCKErrorObjectNotFound = 1,
    
    /// An object was invalid and processing could not continue.
    OCKErrorInvalidObject,
    
    /// An exception was caught during an operation.
    OCKErrorException,
    
    /// Multiple errors were encountered during an operation.
    OCKErrorMultipleErrors
} OCK_ENUM_AVAILABLE;

#if ( defined(OCK_LOG_LEVEL_NONE) && OCK_LOG_LEVEL_NONE )
#  undef OCK_LOG_LEVEL_DEBUG
#  undef OCK_LOG_LEVEL_WARNING
#  undef OCK_LOG_LEVEL_ERROR
#endif

#if ( !defined(OCK_LOG_LEVEL_NONE) && !defined(OCK_LOG_LEVEL_DEBUG) && !defined(OCK_LOG_LEVEL_WARNING) && !defined(OCK_LOG_LEVEL_ERROR) )
#  define OCK_LOG_LEVEL_WARNING 1
#endif

#define _OCK_LogWithLevel(level,fmt,...) NSLog(@"[CareKit]["#level"] %s " fmt, __PRETTY_FUNCTION__, ## __VA_ARGS__)

#if ( OCK_LOG_LEVEL_DEBUG )
#  define OCK_Log_Debug(fmt,...) _OCK_LogWithLevel(Debug, fmt, ## __VA_ARGS__)
#else
#  define OCK_Log_Debug(...)
#endif

#if ( OCK_LOG_LEVEL_DEBUG || OCK_LOG_LEVEL_WARNING )
#  define OCK_Log_Warning(fmt,...) _OCK_LogWithLevel(Warning, fmt, ## __VA_ARGS__)
#else
#  define OCK_Log_Warning(...)
#endif

#if ( OCK_LOG_LEVEL_DEBUG || OCK_LOG_LEVEL_WARNING || OCK_LOG_LEVEL_ERROR )
#  define OCK_Log_Error(fmt,...) _OCK_LogWithLevel(Error, fmt, ## __VA_ARGS__)
#else
#  define OCK_Log_Error(...)
#endif


#if !defined(OCK_INLINE)
#  if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
#    define OCK_INLINE static inline
#  elif defined(__cplusplus)
#    define OCK_INLINE static inline
#  elif defined(__GNUC__)
#    define OCK_INLINE static __inline__
#  else
#    define OCK_INLINE static
#  endif
#endif

#define OCK_NARG(...) OCK_NARG_(__VA_ARGS__,OCK_RSEQ_N())
#define OCK_NARG_(...)  OCK_ARG_N(__VA_ARGS__)
#define OCK_ARG_N( _1, _2, _3, _4, _5, _6, _7, _8, _9,_10, N, ...) N
#define OCK_RSEQ_N()   10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0

#define STRINGIFY2( x) #x
#define STRINGIFY(x) STRINGIFY2(x)

#define OCK_DECODE_OBJ(d,x)  _ ## x = [d decodeObjectForKey:@STRINGIFY(x)]
#define OCK_ENCODE_OBJ(c,x)  [c encodeObject:_ ## x forKey:@STRINGIFY(x)]
#define OCK_ENCODE_URL(c,x)  [c encodeObject:OCKRelativePathForURL(_ ## x) forKey:@STRINGIFY(x)]
#define OCK_ENCODE_URL_BOOKMARK(c, x) [c encodeObject:OCKBookmarkDataFromURL(_ ## x) forKey:@STRINGIFY(x)]

#define OCK_DECODE_OBJ_CLASS(d,x,cl)  _ ## x = (cl *)[d decodeObjectOfClass:[cl class] forKey:@STRINGIFY(x)]
#define OCK_DECODE_OBJ_ARRAY(d,x,cl)  _ ## x = (NSArray *)[d decodeObjectOfClasses:[NSSet setWithObjects:[NSArray class],[cl class],nil] forKey:@STRINGIFY(x)]
#define OCK_DECODE_OBJ_MUTABLE_ORDERED_SET(d,x,cl)  _ ## x = [(NSOrderedSet *)[d decodeObjectOfClasses:[NSSet setWithObjects:[NSOrderedSet class],[cl class],nil] forKey:@STRINGIFY(x)] mutableCopy]
#define OCK_DECODE_OBJ_MUTABLE_DICTIONARY(d,x,kcl,cl)  _ ## x = [(NSDictionary *)[d decodeObjectOfClasses:[NSSet setWithObjects:[NSDictionary class],[kcl class],[cl class],nil] forKey:@STRINGIFY(x)] mutableCopy]

#define OCK_ENCODE_COND_OBJ(c,x)  [c encodeConditionalObject:_ ## x forKey:@STRINGIFY(x)]

#define OCK_DECODE_IMAGE(d,x)  _ ## x = (UIImage *)[d decodeObjectOfClass:[UIImage class] forKey:@STRINGIFY(x)]
#define OCK_ENCODE_IMAGE(c,x)  { if (_ ## x) { UIImage * __ ## x = [UIImage imageWithCGImage:[_ ## x CGImage] scale:[_ ## x scale] orientation:[_ ## x imageOrientation]]; [c encodeObject:__ ## x forKey:@STRINGIFY(x)]; } }

#define OCK_DECODE_URL(d,x) _ ## x = OCKURLForRelativePath((NSString *)[d decodeObjectOfClass:[NSString class] forKey:@STRINGIFY(x)])
#define OCK_DECODE_URL_BOOKMARK(d,x)  _ ## x = OCKURLFromBookmarkData((NSData *)[d decodeObjectOfClass:[NSData class] forKey:@STRINGIFY(x)])

#define OCK_DECODE_BOOL(d,x)  _ ## x = [d decodeBoolForKey:@STRINGIFY(x)]
#define OCK_ENCODE_BOOL(c,x)  [c encodeBool:_ ## x forKey:@STRINGIFY(x)]

#define OCK_DECODE_DOUBLE(d,x)  _ ## x = [d decodeDoubleForKey:@STRINGIFY(x)]
#define OCK_ENCODE_DOUBLE(c,x)  [c encodeDouble:_ ## x forKey:@STRINGIFY(x)]

#define OCK_DECODE_INTEGER(d,x)  _ ## x = [d decodeIntegerForKey:@STRINGIFY(x)]
#define OCK_ENCODE_INTEGER(c,x)  [c encodeInteger:_ ## x forKey:@STRINGIFY(x)]

#define OCK_ENCODE_UINT32(c,x)  [c encodeObject:[NSNumber numberWithUnsignedLongLong:_ ## x] forKey:@STRINGIFY(x)]
#define OCK_DECODE_UINT32(d,x)  _ ## x = (uint32_t)[(NSNumber *)[d decodeObjectForKey:@STRINGIFY(x)] unsignedLongValue]

#define OCK_DECODE_ENUM(d,x)  _ ## x = (__typeof(_ ## x))[d decodeIntegerForKey:@STRINGIFY(x)]
#define OCK_ENCODE_ENUM(c,x)  [c encodeInteger:(NSInteger)_ ## x forKey:@STRINGIFY(x)]

#define OCK_DECODE_CGRECT(d,x)  _ ## x = (__typeof(_ ## x))[d decodeCGRectForKey:@STRINGIFY(x)]
#define OCK_ENCODE_CGRECT(c,x)  [c encodeCGRect:_ ## x forKey:@STRINGIFY(x)]

#define OCK_DECODE_CGSIZE(d,x)  _ ## x = (__typeof(_ ## x))[d decodeCGSizeForKey:@STRINGIFY(x)]
#define OCK_ENCODE_CGSIZE(c,x)  [c encodeCGSize:_ ## x forKey:@STRINGIFY(x)]

#define OCK_DECODE_CGPOINT(d,x)  _ ## x = (__typeof(_ ## x))[d decodeCGPointForKey:@STRINGIFY(x)]
#define OCK_ENCODE_CGPOINT(c,x)  [c encodeCGPoint:_ ## x forKey:@STRINGIFY(x)]

#define OCK_DECODE_UIEDGEINSETS(d,x)  _ ## x = (__typeof(_ ## x))[d decodeUIEdgeInsetsForKey:@STRINGIFY(x)]
#define OCK_ENCODE_UIEDGEINSETS(c,x)  [c encodeUIEdgeInsets:_ ## x forKey:@STRINGIFY(x)]

#define OCK_DECODE_COORDINATE(d,x)  _ ## x = CLLocationCoordinate2DMake([d decodeDoubleForKey:@STRINGIFY(x.latitude)],[d decodeDoubleForKey:@STRINGIFY(x.longitude)])
#define OCK_ENCODE_COORDINATE(c,x)  [c encodeDouble:_ ## x.latitude forKey:@STRINGIFY(x.latitude)];[c encodeDouble:_ ## x.longitude forKey:@STRINGIFY(x.longitude)];

/*
 * Helpers for completions which call the block only if non-nil
 *
 */
#define BLOCK_EXEC(block, ...) if (block) { block(__VA_ARGS__); };

#define DISPATCH_EXEC(queue, block, ...) if (block) { dispatch_async(queue, ^{ block(__VA_ARGS__); } ); }

/*
 * For testing background delivery
 *
 */
#if OCK_BACKGROUND_DELIVERY_TEST
#  define OCK_HEALTH_UPDATE_FREQUENCY HKUpdateFrequencyImmediate
#else
#  define OCK_HEALTH_UPDATE_FREQUENCY HKUpdateFrequencyDaily
#endif

// Find the first object of the specified class, using method as the iterator
#define OCKFirstObjectOfClass(C,p,method) ({ id v = p; while (v != nil) { if ([v isKindOfClass:[C class]]) { break; } else { v = [v method]; } }; v; })

// Bundle for video assets
NSBundle *OCKAssetsBundle(void);
NSBundle *OCKBundle(void);
NSBundle *OCKDefaultLocaleBundle(void);

// Pass 0xcccccc and get color #cccccc
UIColor *OCKRGB(uint32_t x);
UIColor *OCKRGBA(uint32_t x, CGFloat alpha);
UIColor *OCKSystemGrayColor(void);

NSString *OCKStringFromDateISO8601(NSDate *date);
NSDate *OCKDateFromStringISO8601(NSString *string);

NSString *OCKSignatureStringFromDate(NSDate *date);

NSURL *OCKCreateRandomBaseURL(void);

CGFloat OCKExpectedLabelHeight(UILabel *label);
void OCKAdjustHeightForLabel(UILabel *label);

// build a image with color
UIImage *OCKImageWithColor(UIColor *color);

void OCKEnableAutoLayoutForViews(NSArray *views);

NSDateComponentsFormatter *OCKTimeIntervalLabelFormatter(void);
NSDateComponentsFormatter *OCKDurationStringFormatter(void);
NSNumberFormatter *OCKPercentFormatter(NSInteger maxFractionDigits, NSInteger minFractionDigits);

NSDateFormatter *OCKTimeOfDayLabelFormatter(void);
NSCalendar *OCKTimeOfDayReferenceCalendar(void);

NSDateComponents *OCKTimeOfDayComponentsFromDate(NSDate *date);
NSDate *OCKTimeOfDayDateFromComponents(NSDateComponents *dateComponents);

BOOL OCKCurrentLocalePresentsFamilyNameFirst(void);

UIFont *OCKTimeFontForSize(CGFloat size);
UIFontDescriptor *OCKFontDescriptorForLightStylisticAlternative(UIFontDescriptor *descriptor);

CGFloat OCKFloorToViewScale(CGFloat value, UIView *view);

OCK_INLINE bool
OCKEqualObjects(id o1, id o2) {
    return (o1 == o2) || (o1 && o2 && [o1 isEqual:o2]);
}

OCK_INLINE BOOL
OCKEqualFileURLs(NSURL *url1, NSURL *url2) {
    return OCKEqualObjects(url1, url2) || ([url1 isFileURL] && [url2 isFileURL] && [[url1 absoluteString] isEqualToString:[url2 absoluteString]]);
}

OCK_INLINE NSArray *
OCKArrayCopyObjects(NSArray *a) {
    if (!a) {
        return nil;
    }
    NSMutableArray *b = [NSMutableArray arrayWithCapacity:a.count];
    [a enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        [b addObject:[obj copy]];
    }];
    return [b copy];
}

OCK_INLINE NSMutableOrderedSet *
OCKMutableOrderedSetCopyObjects(NSOrderedSet *a) {
    if (!a) {
        return nil;
    }
    NSMutableOrderedSet *b = [NSMutableOrderedSet orderedSetWithCapacity:a.count];
    [a enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        [b addObject:[obj copy]];
    }];
    return b;
}

OCK_INLINE NSMutableDictionary *
OCKMutableDictionaryCopyObjects(NSDictionary *a) {
    if (!a) {
        return nil;
    }
    NSMutableDictionary *b = [NSMutableDictionary dictionaryWithCapacity:a.count];
    [a enumerateKeysAndObjectsUsingBlock:^(id key, id obj, BOOL *stop) {
        b[key] = [obj copy];
    }];
    return b;
}

#define OCKSuppressPerformSelectorWarning(PerformCall) \
do { \
_Pragma("clang diagnostic push") \
_Pragma("clang diagnostic ignored \"-Warc-performSelector-leaks\"") \
PerformCall; \
_Pragma("clang diagnostic pop") \
} while (0)

UIFont *OCKThinFontWithSize(CGFloat size);
UIFont *OCKLightFontWithSize(CGFloat size);
UIFont *OCKMediumFontWithSize(CGFloat size);

NSURL *OCKURLFromBookmarkData(NSData *data);
NSData *OCKBookmarkDataFromURL(NSURL *url);

NSString *OCKPathRelativeToURL(NSURL *url, NSURL *baseURL);
NSURL *OCKURLForRelativePath(NSString *relativePath);
NSString *OCKRelativePathForURL(NSURL *url);

id OCKDynamicCast_(id x, Class objClass);

#define OCKDynamicCast(x, c) ((c *) OCKDynamicCast_(x, [c class]))

OCK_INLINE CGFloat
OCKCGFloatNearlyEqualToFloat(CGFloat f1, CGFloat f2) {
    const CGFloat OCKCGFloatEpsilon = 0.01; // 0.01 should be safe enough when dealing with screen point and pixel values
    return (ABS(f1 - f2) <= OCKCGFloatEpsilon);
}
#define OCKDefineStringKey(x) static NSString *const x = @STRINGIFY(x)

#define OCKThrowMethodUnavailableException()  @throw [NSException exceptionWithName:NSInternalInconsistencyException reason:@"method unavailable" userInfo:nil];
#define OCKThrowInvalidArgumentExceptionIfNil(argument)  if (!argument) { @throw [NSException exceptionWithName:NSInvalidArgumentException reason:@#argument" cannot be nil." userInfo:nil]; }

void OCKValidateArrayForObjectsOfClass(NSArray *array, Class expectedObjectClass, NSString *exceptionReason);

void OCKRemoveConstraintsForRemovedViews(NSMutableArray *constraints, NSArray *removedViews);

void OCKAdjustPageViewControllerNavigationDirectionForRTL(UIPageViewControllerNavigationDirection *direction);

NSString *OCKPaddingWithNumberOfSpaces(NSUInteger numberOfPaddingSpaces);

NSString *OCKStripNonNumericCharacters(NSString *string);

#define OCKAccessibilityStringForVariables(firstString, ...) _OCKAccessibilityStringForVariables(firstString, ##__VA_ARGS__, __AXStringForVariablesSentinel)
OCK_EXTERN NSString *const __AXStringForVariablesSentinel;
NSString *_OCKAccessibilityStringForVariables(id firstArgument, ...);
